/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998-2000  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * lfd_encrypt.c,v 1.2.2.6 2002/04/25 09:19:50 bergolth Exp
 */ 


/*
 * This lfd_encrypt module uses MD5 to create 128 bits encryption
 * keys and BlowFish for actual data encryption.
 * It is based on code written by Chris Todd<christ@insynq.com> with 
 * several improvements and modifications by me.  
 */

#include "config.h"

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <strings.h>
#include <string.h>

#include "vtun.h"
#include "linkfd.h"
#include "lib.h"

#ifdef HAVE_SSL

#include "Schrack_AES.h"

#define ENC_BUF_SIZE VTUN_FRAME_SIZE + 16 
#define ENC_KEY_SIZE 16


// legacy
char * enc_buf;

int Crypt_Alloc( struct vtun_host* host )
{
   // Crypting in-place for now, so no need for separate buffer
   /*if( !(enc_buf = lfd_alloc(ENC_BUF_SIZE)) ){
      vtun_syslog(LOG_ERR,"Can't allocate buffer for encryptor");
      return -1;
   }*/

   AESGCM_Init( host->passwd, strlen( host->passwd ) );
   vtun_syslog( LOG_INFO, "AES-GCM encryption initialized");
   return 0;
}

int Crypt_Free( void )
{
   //lfd_free(enc_buf); enc_buf = NULL;
   AESGCM_Exit( );
   return 0;
}

int Crypt_EncryptBuffer( int len, char* in, char** out )
{
   return AESGCM_Encrypt( in, len, out );
}


// Decrypt the given buffer and return length.
int Crypt_DecryptBuffer( int len, char* in, char** out )
{
   return AESGCM_DecryptAuth( in, len, out );
}

/* 
 * Module structure.
 */
struct lfd_mod lfd_encrypt = {
     "Encryptor",
     Crypt_Alloc,
     Crypt_EncryptBuffer,
     NULL,
     Crypt_DecryptBuffer,
     NULL,
     Crypt_Free,
     NULL,
     NULL
};

#else  /* HAVE_SSL */

int no_encrypt(struct vtun_host *host)
{
     vtun_syslog(LOG_INFO, "Encryption is not supported");
     return -1;
}

struct lfd_mod lfd_encrypt = {
     "Encryptor",
     no_encrypt, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};

#endif /* HAVE_SSL */
